@@ -19,7 +19,7 @@
 // Modified by Anders Logg 2008-2013
 //
 // First added:  2007-05-24
-// Last changed: 2013-09-24
+// Last changed: 2014-04-25
 
 #ifndef __SPARSITY_PATTERN_BUILDER_H
 #define __SPARSITY_PATTERN_BUILDER_H
@@ -32,30 +32,41 @@ namespace dolfin
 {
 
   class GenericDofMap;
-  class GenericSparsityPattern;
   class Mesh;
-  class CCFEMForm;
+  class MultiMeshForm;
+  class SparsityPattern;
 
-  /// This class provides functions to compute the sparsity pattern.
+  /// This class provides functions to compute the sparsity pattern
+  /// based on DOF maps
 
   class SparsityPatternBuilder
   {
   public:
 
     /// Build sparsity pattern for assembly of given form
-    static void build(GenericSparsityPattern& sparsity_pattern,
+    static void build(SparsityPattern& sparsity_pattern,
                       const Mesh& mesh,
                       const std::vector<const GenericDofMap*> dofmaps,
                       bool cells,
                       bool interior_facets,
                       bool exterior_facets,
+                      bool vertices,
                       bool diagonal,
                       bool init=true,
                       bool finalize=true);
 
-    /// Build sparsity pattern for assembly of given CCFEM form
-    static void build_ccfem(GenericSparsityPattern& sparsity_pattern,
-                            const CCFEMForm& form);
+    /// Build sparsity pattern for assembly of given multimesh form
+    static void
+      build_multimesh_sparsity_pattern(SparsityPattern& sparsity_pattern,
+                                       const MultiMeshForm& form);
+
+  private:
+
+    // Build sparsity pattern for interface part of multimesh form
+    static void _build_multimesh_sparsity_pattern_interface
+      (SparsityPattern& sparsity_pattern,
+       const MultiMeshForm& form,
+       std::size_t part);
 
   };
 
