@@ -1,4 +1,4 @@
-// Copyright (C) 2006-2013 Anders Logg
+// Copyright (C) 2006-2016 Anders Logg
 //
 // This file is part of DOLFIN.
 //
@@ -32,7 +32,6 @@
 #include <string>
 #include <utility>
 
-#include <dolfin/ale/MeshDisplacement.h>
 #include <dolfin/common/Hierarchical.h>
 #include <dolfin/common/MPI.h>
 #include <dolfin/common/Variable.h>
@@ -60,21 +59,13 @@ namespace dolfin
   /// dimension-independent, but a concrete interface is also provided
   /// for standard named mesh entities:
   ///
-  /// .. tabularcolumns:: |c|c|c|
-  ///
-  /// +--------+-----------+-------------+
-  /// | Entity | Dimension | Codimension |
-  /// +========+===========+=============+
-  /// | Vertex |  0        |             |
-  /// +--------+-----------+-------------+
-  /// | Edge   |  1        |             |
-  /// +--------+-----------+-------------+
-  /// | Face   |  2        |             |
-  /// +--------+-----------+-------------+
-  /// | Facet  |           |      1      |
-  /// +--------+-----------+-------------+
-  /// | Cell   |           |      0      |
-  /// +--------+-----------+-------------+
+  /// | Entity | Dimension | Codimension  |
+  /// | ------ | --------- | ------------ |
+  /// | Vertex |  0        |              |
+  /// | Edge   |  1        |              |
+  /// | Face   |  2        |              |
+  /// | Facet  |           |      1       |
+  /// | Cell   |           |      0       |
   ///
   /// When working with mesh iterators, all entities and connectivity
   /// are precomputed automatically the first time an iterator is
@@ -100,33 +91,29 @@ namespace dolfin
 
     /// Copy constructor.
     ///
-    /// *Arguments*
-    ///     mesh (_Mesh_)
+    /// @param mesh (Mesh)
     ///         Object to be copied.
     Mesh(const Mesh& mesh);
 
     /// Create mesh from data file.
     ///
-    /// *Arguments*
-    ///     filename (std::string)
+    /// @param  filename (std::string)
     ///         Name of file to load.
     explicit Mesh(std::string filename);
 
     /// Create mesh from data file.
     ///
-    /// *Arguments*
-    ///     comm (MPI_Comm)
+    /// @param comm (MPI_Comm)
     ///         The MPI communicator
-    ///     filename (std::string)
+    /// @param filename (std::string)
     ///         Name of file to load.
     Mesh(MPI_Comm comm, std::string filename);
 
     /// Create a distributed mesh from local (per process) data.
     ///
-    /// *Arguments*
-    ///     comm (MPI_Comm)
+    /// @param comm (MPI_Comm)
     ///         MPI communicator for the mesh.
-    ///     local_mesh_data (_LocalMeshData_)
+    /// @param local_mesh_data (LocalMeshData)
     ///         Data from which to build the mesh.
     Mesh(MPI_Comm comm, LocalMeshData& local_mesh_data);
 
@@ -135,190 +122,135 @@ namespace dolfin
 
     /// Assignment operator
     ///
-    /// *Arguments*
-    ///     mesh (_Mesh_)
-    ///         Another _Mesh_ object.
+    /// @param mesh (Mesh)
+    ///         Another Mesh object.
     const Mesh& operator=(const Mesh& mesh);
 
     /// Get number of vertices in mesh.
     ///
-    /// *Returns*
-    ///     std::size_t
+    /// @return std::size_t
     ///         Number of vertices.
     ///
-    /// *Example*
-    ///     .. note::
-    ///
-    ///         No example code available for this function.
     std::size_t num_vertices() const
     { return _topology.size(0); }
 
     /// Get number of edges in mesh.
     ///
-    /// *Returns*
-    ///     std::size_t
+    /// @return std::size_t
     ///         Number of edges.
     ///
-    /// *Example*
-    ///     .. note::
-    ///
-    ///         No example code available for this function.
     std::size_t num_edges() const
     { return _topology.size(1); }
 
     /// Get number of faces in mesh.
     ///
-    /// *Returns*
-    ///     std::size_t
+    /// @return std::size_t
     ///         Number of faces.
     ///
-    /// *Example*
-    ///     .. note::
-    ///
-    ///         No example code available for this function.
     std::size_t num_faces() const
     { return _topology.size(2); }
 
     /// Get number of facets in mesh.
     ///
-    /// *Returns*
-    ///     std::size_t
+    /// @return std::size_t
     ///         Number of facets.
     ///
-    /// *Example*
-    ///     .. note::
-    ///
-    ///         No example code available for this function.
     std::size_t num_facets() const
     { return _topology.size(_topology.dim() - 1); }
 
     /// Get number of cells in mesh.
     ///
-    /// *Returns*
-    ///     std::size_t
+    /// @return std::size_t
     ///         Number of cells.
     ///
-    /// *Example*
-    ///     .. note::
-    ///
-    ///         No example code available for this function.
     std::size_t num_cells() const
     { return _topology.size(_topology.dim()); }
 
     /// Get number of entities of given topological dimension.
     ///
-    /// *Arguments*
-    ///     d (std::size_t)
+    /// @param d (std::size_t)
     ///         Topological dimension.
     ///
-    /// *Returns*
-    ///     std::size_t
+    /// @return std::size_t
     ///         Number of entities of topological dimension d.
     ///
-    /// *Example*
-    ///     .. note::
-    ///
-    ///         No example code available for this function.
     std::size_t num_entities(std::size_t d) const
     { return _topology.size(d); }
 
     /// Get vertex coordinates.
     ///
-    /// *Returns*
-    ///     std::vector<double>&
+    /// @return std::vector<double>&
     ///         Coordinates of all vertices.
     ///
-    /// *Example*
-    ///     .. note::
-    ///
-    ///         No example code available for this function.
     std::vector<double>& coordinates()
     { return _geometry.x(); }
 
     /// Return coordinates of all vertices (const version).
+    ///
+    /// @return std::vector<double>&
+    ///         Coordinates of all vertices.
+    ///
     const std::vector<double>& coordinates() const
     { return _geometry.x(); }
 
     /// Get cell connectivity.
     ///
-    /// *Returns*
-    ///     std::vector<std::size_t>
+    /// @return std::vector<unsigned int>&
     ///         Connectivity for all cells.
     ///
-    /// *Example*
-    ///     .. note::
-    ///
-    ///         No example code available for this function.
     const std::vector<unsigned int>& cells() const
     { return _topology(_topology.dim(), 0)(); }
 
-    /// Get number of local entities of given topological dimension.
-    ///
-    /// *Arguments*
-    ///     dim (std::size_t)
-    ///         Topological dimension.
-    ///
-    /// *Returns*
-    ///     std::size_t
-    ///         Number of local entities of topological dimension d.
-    ///
-    /// *Example*
-    ///     .. note::
-    ///
-    ///         No example code available for this function.
-    std::size_t size(std::size_t dim) const
-    { return _topology.size(dim); }
-
     /// Get global number of entities of given topological dimension.
     ///
-    /// *Arguments*
-    ///     dim (std::size_t)
+    /// @param dim (std::size_t)
     ///         Topological dimension.
     ///
-    /// *Returns*
-    ///     std::size_t
+    /// @return std::size_t
     ///         Global number of entities of topological dimension d.
     ///
-    /// *Example*
-    ///     .. note::
-    ///
-    ///         No example code available for this function.
-    std::size_t size_global(std::size_t dim) const
+    std::size_t num_entities_global(std::size_t dim) const
     { return _topology.size_global(dim); }
 
     /// Get mesh topology.
     ///
-    /// *Returns*
-    ///     _MeshTopology_
+    /// @return MeshTopology
     ///         The topology object associated with the mesh.
     MeshTopology& topology()
     { return _topology; }
 
     /// Get mesh topology (const version).
+    ///
+    /// @return MeshTopology
+    ///         The topology object associated with the mesh.
     const MeshTopology& topology() const
     { return _topology; }
 
     /// Get mesh geometry.
     ///
-    /// *Returns*
-    ///     _MeshGeometry_
+    /// @return MeshGeometry
     ///         The geometry object associated with the mesh.
     MeshGeometry& geometry()
     { return _geometry; }
 
     /// Get mesh geometry (const version).
+    ///
+    /// @return MeshGeometry
+    ///         The geometry object associated with the mesh.
     const MeshGeometry& geometry() const
     { return _geometry; }
 
     /// Get mesh (sub)domains.
     ///
-    /// *Returns*
-    ///     _MeshDomains_
+    /// @return MeshDomains
     ///         The (sub)domains associated with the mesh.
     MeshDomains& domains()
     { return _domains; }
 
-    /// Get mesh (sub)domains.
+    /// Get mesh (sub)domains (const).
+    ///
+    /// @return MeshDomains
+    ///         The (sub)domains associated with the mesh.
     const MeshDomains& domains() const
     { return _domains; }
 
@@ -329,22 +261,24 @@ namespace dolfin
     /// responsibility of the caller to use (and possibly rebuild) the
     /// tree. It is stored as a (mutable) member of the mesh to enable
     /// sharing of the bounding box tree data structure.
+    ///
+    /// @return std::shared_ptr<BoundingBoxTree>
     std::shared_ptr<BoundingBoxTree> bounding_box_tree() const;
 
     /// Get mesh data.
     ///
-    /// *Returns*
-    ///     _MeshData_
+    /// @return MeshData&
     ///         The mesh data object associated with the mesh.
     MeshData& data();
 
     /// Get mesh data (const version).
+    /// @return MeshData&
+    ///         The mesh data object associated with the mesh.
     const MeshData& data() const;
 
     /// Get mesh cell type.
     ///
-    /// *Returns*
-    ///     _CellType_
+    /// @return CellType&
     ///         The cell type object associated with the mesh.
     CellType& type()
     { dolfin_assert(_cell_type); return *_cell_type; }
@@ -355,22 +289,19 @@ namespace dolfin
 
     /// Compute entities of given topological dimension.
     ///
-    /// *Arguments*
-    ///     dim (std::size_t)
+    /// @param  dim (std::size_t)
     ///         Topological dimension.
     ///
-    /// *Returns*
-    ///     std::size_t
+    /// @return std::size_t
     ///         Number of created entities.
     std::size_t init(std::size_t dim) const;
 
     /// Compute connectivity between given pair of dimensions.
     ///
-    /// *Arguments*
-    ///     d0 (std::size_t)
+    /// @param    d0 (std::size_t)
     ///         Topological dimension.
     ///
-    ///     d1 (std::size_t)
+    /// @param    d1 (std::size_t)
     ///         Topological dimension.
     void init(std::size_t d0, std::size_t d1) const;
 
@@ -387,15 +318,12 @@ namespace dolfin
 
     /// Order all mesh entities.
     ///
-    /// .. seealso::
-    ///
-    ///     UFC documentation (put link here!)
+    /// See also: UFC documentation (put link here!)
     void order();
 
     /// Check if mesh is ordered according to the UFC numbering convention.
     ///
-    /// *Returns*
-    ///     bool
+    /// @return bool
     ///         The return values is true iff the mesh is ordered.
     bool ordered() const;
 
@@ -405,52 +333,55 @@ namespace dolfin
     /// renumbered to improve the locality within each color. It is
     /// assumed that the mesh has already been colored and that only
     /// cell-vertex connectivity exists as part of the mesh.
+    /// @return Mesh
     Mesh renumber_by_color() const;
 
+    /// Scale mesh coordinates with given factor.
+    ///
+    /// *Arguments*
+    ///     factor (double)
+    ///         The factor defining the scaling.
+    void scale(double factor);
+
     /// Translate mesh according to a given vector.
     ///
-    /// *Arguments*
-    ///     point (Point)
+    /// @param  point (Point)
     ///         The vector defining the translation.
     void translate(const Point& point);
 
     /// Rotate mesh around a coordinate axis through center of mass
     /// of all mesh vertices
     ///
-    /// *Arguments*
-    ///     angle (double)
+    /// @param    angle (double)
     ///         The number of degrees (0-360) of rotation.
-    ///     axis (std::size_t)
+    /// @param    axis (std::size_t)
     ///         The coordinate axis around which to rotate the mesh.
     void rotate(double angle, std::size_t axis=2);
 
     /// Rotate mesh around a coordinate axis through a given point
     ///
-    /// *Arguments*
-    ///     angle (double)
+    /// @param    angle (double)
     ///         The number of degrees (0-360) of rotation.
-    ///     axis (std::size_t)
+    /// @param    axis (std::size_t)
     ///         The coordinate axis around which to rotate the mesh.
-    ///     point (_Point_)
+    /// @param    point (_Point_)
     ///         The point around which to rotate the mesh.
     void rotate(double angle, std::size_t axis, const Point& point);
 
     /// Smooth internal vertices of mesh by local averaging.
     ///
-    /// *Arguments*
-    ///     num_iterations (std::size_t)
+    /// @param    num_iterations (std::size_t)
     ///         Number of iterations to perform smoothing,
     ///         default value is 1.
     void smooth(std::size_t num_iterations=1);
 
     /// Smooth boundary vertices of mesh by local averaging.
     ///
-    /// *Arguments*
-    ///     num_iterations (std::size_t)
+    /// @param  num_iterations (std::size_t)
     ///         Number of iterations to perform smoothing,
     ///         default value is 1.
     ///
-    ///     harmonic_smoothing (bool)
+    /// @param  harmonic_smoothing (bool)
     ///         Flag to turn on harmonics smoothing, default
     ///         value is true.
     void smooth_boundary(std::size_t num_iterations=1,
@@ -458,11 +389,10 @@ namespace dolfin
 
     /// Snap boundary vertices of mesh to match given sub domain.
     ///
-    /// *Arguments*
-    ///     sub_domain (_SubDomain_)
+    /// @param sub_domain (_SubDomain_)
     ///         A _SubDomain_ object.
     ///
-    ///     harmonic_smoothing (bool)
+    /// @param harmonic_smoothing (bool)
     ///         Flag to turn on harmonics smoothing, default
     ///         value is true.
     void snap_boundary(const SubDomain& sub_domain,
@@ -470,32 +400,28 @@ namespace dolfin
 
     /// Color the cells of the mesh such that no two neighboring cells
     /// share the same color. A colored mesh keeps a
-    /// CellFunction<std::size_t> named "cell colors" as mesh data which
+    /// MeshFunction<std::size_t> named "cell colors" as mesh data which
     /// holds the colors of the mesh.
     ///
-    /// *Arguments*
-    ///     coloring_type (std::string)
+    /// @param coloring_type (std::string)
     ///         Coloring type, specifying what relation makes two
     ///         cells neighbors, can be one of "vertex", "edge" or
     ///         "facet".
     ///
-    /// *Returns*
-    ///     std::vector<std::size_t>
+    /// @return std::vector<std::size_t>&
     ///         The colors as a mesh function over the cells of the mesh.
     const std::vector<std::size_t>& color(std::string coloring_type) const;
 
     /// Color the cells of the mesh such that no two neighboring cells
     /// share the same color. A colored mesh keeps a
-    /// CellFunction<std::size_t> named "cell colors" as mesh data which
+    /// MeshFunction<std::size_t> named "cell colors" as mesh data which
     /// holds the colors of the mesh.
     ///
-    /// *Arguments*
-    ///     coloring_type (std::vector<std::size_t>)
+    /// @param coloring_type (std::vector<std::size_t>&)
     ///         Coloring type given as list of topological dimensions,
     ///         specifying what relation makes two mesh entities neighbors.
     ///
-    /// *Returns*
-    ///     std::vector<std::size_t>
+    /// @return std::vector<std::size_t>&
     ///         The colors as a mesh function over entities of the mesh.
     const std::vector<std::size_t>&
     color(std::vector<std::size_t> coloring_type) const;
@@ -503,85 +429,57 @@ namespace dolfin
     /// Compute minimum cell size in mesh, measured greatest distance
     /// between any two vertices of a cell.
     ///
-    /// *Returns*
-    ///     double
+    /// @return double
     ///         The minimum cell size. The size is computed using
     ///         Cell::h()
     ///
-    /// *Example*
-    ///     .. note::
-    ///
-    ///         No example code available for this function.
     double hmin() const;
 
     /// Compute maximum cell size in mesh, measured greatest distance
     /// between any two vertices of a cell.
     ///
-    /// *Returns*
-    ///     double
+    /// @return double
     ///         The maximum cell size. The size is computed using
     ///         Cell::h()
     ///
-    /// *Example*
-    ///     .. note::
-    ///
-    ///         No example code available for this function.
     double hmax() const;
 
     /// Compute minimum cell inradius.
     ///
-    /// *Returns*
-    ///     double
+    /// @return double
     ///         The minimum of cells' inscribed sphere radii
     ///
-    /// *Example*
-    ///     .. note::
-    ///
-    ///         No example code available for this function.
     double rmin() const;
 
     /// Compute maximum cell inradius.
     ///
-    /// *Returns*
-    ///     double
+    /// @return double
     ///         The maximum of cells' inscribed sphere radii
     ///
-    /// *Example*
-    ///     .. note::
-    ///
-    ///         No example code available for this function.
     double rmax() const;
 
     /// Compute hash of mesh, currently based on the has of the mesh
     /// geometry and mesh topology.
     ///
-    /// *Returns*
-    ///     std::size_t
+    /// @return std::size_t
     ///         A tree-hashed value of the coordinates over all MPI processes
     ///
     std::size_t hash() const;
 
     /// Informal string representation.
     ///
-    /// *Arguments*
-    ///     verbose (bool)
+    /// @param verbose (bool)
     ///         Flag to turn on additional output.
     ///
-    /// *Returns*
-    ///     std::string
+    /// @return std::string
     ///         An informal representation of the mesh.
     ///
-    /// *Example*
-    ///     .. note::
-    ///
-    ///         No example code available for this function.
     std::string str(bool verbose) const;
 
     /// Return cell_orientations (const version)
     ///
-    /// *Returns*
-    ///     std::vector<int>
-
+    /// @return std::vector<int>
+    ///
     ///         Map from cell index to orientation of cell. Is empty
     ///         if cell orientations have not been computed.
     const std::vector<int>& cell_orientations() const;
@@ -590,23 +488,31 @@ namespace dolfin
     /// global outward direction/normal/orientation. Only defined if
     /// mesh is orientable.
     ///
-    /// *Arguments*
-    ///     global_normal (Expression)
+    /// @param global_normal (Expression)
     ///         A global normal direction to the mesh
     void init_cell_orientations(const Expression& global_normal);
 
     /// Mesh MPI communicator
+    /// @return MPI_Comm
     MPI_Comm mpi_comm() const
-    { return _mpi_comm; }
+    { return _mpi_comm.comm(); }
+
+    /// Ghost mode used for partitioning. Possible values are
+    /// same as `parameters["ghost_mode"]`.
+    /// WARNING: the interface may change in future without
+    /// deprecation; the method is now intended for internal
+    /// library use.
+    std::string ghost_mode() const;
 
     // Friend in fem_utils.h
-    friend Mesh create_mesh(Function&);
+    friend Mesh create_mesh(Function& coordinates);
 
   private:
 
     // Friends
     friend class MeshEditor;
     friend class TopologyComputation;
+    friend class MeshPartitioning;
 
     // Mesh topology
     MeshTopology _topology;
@@ -635,7 +541,10 @@ namespace dolfin
     std::vector<int> _cell_orientations;
 
     // MPI communicator
-    MPI_Comm _mpi_comm;
+    dolfin::MPI::Comm _mpi_comm;
+
+    // Ghost mode used for partitioning
+    std::string _ghost_mode;
 
   };
 }
