@@ -34,11 +34,14 @@
 #include <dolfin/fem/assemble.h>
 #include <dolfin/fem/assemble_local.h>
 #include <dolfin/fem/Assembler.h>
+#include <dolfin/fem/MultiMeshAssembler.h>
 #include <dolfin/fem/DirichletBC.h>
 #include <dolfin/fem/DiscreteOperators.h>
 #include <dolfin/fem/DofMap.h>
+#include <dolfin/fem/MultiMeshDofMap.h>
 #include <dolfin/fem/FiniteElement.h>
 #include <dolfin/fem/Form.h>
+#include <dolfin/fem/MultiMeshForm.h>
 #include <dolfin/fem/LinearVariationalProblem.h>
 #include <dolfin/fem/LinearVariationalSolver.h>
 #include <dolfin/fem/LocalSolver.h>
@@ -50,13 +53,19 @@
 #include <dolfin/fem/SystemAssembler.h>
 #include <dolfin/function/GenericFunction.h>
 #include <dolfin/function/FunctionSpace.h>
+#include <dolfin/function/MultiMeshFunctionSpace.h>
 #include <dolfin/function/Function.h>
+#include <dolfin/function/MultiMeshFunction.h>
 #include <dolfin/la/GenericMatrix.h>
 #include <dolfin/la/GenericVector.h>
 #include <dolfin/la/GenericTensor.h>
 #include <dolfin/la/SparsityPattern.h>
 #include <dolfin/mesh/Mesh.h>
+#include <dolfin/mesh/MeshFunction.h>
+#include <dolfin/mesh/MultiMesh.h>
+#include <dolfin/fem/MultiMeshDirichletBC.h>
 #include <dolfin/mesh/SubDomain.h>
+#include <dolfin/adaptivity/adapt.h>
 
 #include "casters.h"
 
@@ -74,6 +83,14 @@ namespace dolfin_wrappers
     py::class_<ufc::form, std::shared_ptr<ufc::form>>
       (m, "ufc_form", "UFC form object");
 
+    // Adapt functions
+    m.def("adapt", [](dolfin::Mesh mesh){return dolfin::adapt(mesh);},
+	  "Mapping between parent and child entities has to be set in refined mesh. See various refinement algorithms and `parameters['refinement_algorithm']`.");
+    m.def("adapt", [](const dolfin::MeshFunction<std::size_t>& mesh_function,
+		      std::shared_ptr<const dolfin::Mesh> adapted_mesh)
+	  {
+	    return dolfin::adapt(mesh_function, adapted_mesh);});
+
     // Function to convert pointers (from JIT usually) to UFC objects
     m.def("make_ufc_finite_element",
           [](std::uintptr_t e)
@@ -146,7 +163,20 @@ namespace dolfin_wrappers
              self.evaluate_basis(i, values.mutable_data(), x.data(), coordinate_dofs.data(),
                                  cell_orientation);
              return values;
-           })
+           }, "Evaluate basis function i at given point x in cell.")
+     .def("evaluate_basis_all", [](const dolfin::FiniteElement& self,
+                               const py::array_t<double> x,
+                               const py::array_t<double> coordinate_dofs,
+                               int cell_orientation)
+          {
+            auto ufc_element = self.ufc_element();
+            const std::size_t size = ufc_element->value_size();
+            const std::size_t space_dimension = ufc_element->space_dimension();
+            py::array_t<double, py::array::c_style> values(space_dimension*size);
+            self.evaluate_basis_all(values.mutable_data(), x.data(), coordinate_dofs.data(),
+                                cell_orientation);
+            return values;
+          }, "Evaluate order n derivatives of all basis functions at given point x in cell.")
       .def("evaluate_basis_derivatives", [](const dolfin::FiniteElement& self,
                                             int i, int order,
                                             const py::array_t<double> x,
@@ -163,7 +193,26 @@ namespace dolfin_wrappers
                                              x.data(), coordinate_dofs.data(),
                                              cell_orientation);
              return values;
-           })
+           }, "Evaluate order n derivatives of basis function i at given point x in cell.")
+      .def("evaluate_basis_derivatives_all", [](const dolfin::FiniteElement& self,
+                                            int order,
+                                            const py::array_t<double> x,
+                                            const py::array_t<double> coordinate_dofs,
+                                            int cell_orientation)
+          {
+            auto ufc_element = self.ufc_element();
+
+            const std::size_t gdim = self.geometric_dimension();
+            const std::size_t num_derivs = pow(gdim, order);
+            const std::size_t size = ufc_element->value_size()*num_derivs;
+            const std::size_t space_dimension = ufc_element->space_dimension();
+            py::array_t<double, py::array::c_style> values(size*space_dimension);
+            self.evaluate_basis_derivatives_all(order, values.mutable_data(),
+                                            x.data(), coordinate_dofs.data(),
+                                            cell_orientation);
+            return values;
+          }, "Evaluate all basis functions at given point x in cell.")
+
       .def("space_dimension", &dolfin::FiniteElement::space_dimension)
       .def("geometric_dimension", &dolfin::FiniteElement::geometric_dimension)
       .def("value_dimension", &dolfin::FiniteElement::value_dimension)
@@ -231,6 +280,12 @@ namespace dolfin_wrappers
       .def("ownership_range", &dolfin::DofMap::ownership_range)
       .def("cell_dofs", &dolfin::DofMap::cell_dofs);
 
+    // dolfin::MultiMeshDofMap
+    py::class_<dolfin::MultiMeshDofMap, std::shared_ptr<dolfin::MultiMeshDofMap>>
+      (m, "MultiMeshDofMap", "DOLFIN MultiMeshDofMap object")
+      .def(py::init())
+      .def("inactive_dofs", &dolfin::MultiMeshDofMap::inactive_dofs);
+
     // dolfin::SparsityPatternBuilder
     py::class_<dolfin::SparsityPatternBuilder>(m, "SparsityPatternBuilder")
       .def_static("build", &dolfin::SparsityPatternBuilder::build,
@@ -283,11 +338,63 @@ namespace dolfin_wrappers
            {
              auto _u = value.attr("_cpp_object").cast<std::shared_ptr<const dolfin::GenericFunction>>();
              self.set_value(_u);
-           });
+           })
+      .def("value", &dolfin::DirichletBC::value);
+
+
+    // dolfin::MultiMeshDirichletBC
+    py::class_<dolfin::MultiMeshDirichletBC, std::shared_ptr<dolfin::MultiMeshDirichletBC>>
+      (m, "MultiMeshDirichletBC", "DOLFIN MultiMeshDirichletBC object")
+      .def(py::init<const dolfin::MultiMeshDirichletBC&>())
+      // Need to handle FunctionSpace and SubDomain coming from the Python layer
+      // Probably better to make Python class for it similar to DirichletBC
+      .def(py::init([](std::shared_ptr<const dolfin::MultiMeshFunctionSpace> V,
+                       py::object g, std::shared_ptr<const dolfin::SubDomain> subdomain,
+                       std::string method, bool check_midpoint,
+                       bool exclude_overlapped_boundaries)
+                    {
+                      std::shared_ptr<const dolfin::GenericFunction> _g;
+                      if (py::hasattr(g, "_cpp_object"))
+                        _g = g.attr("_cpp_object").cast<std::shared_ptr<dolfin::GenericFunction>>();
+                      else
+                        _g = g.cast<std::shared_ptr<dolfin::GenericFunction>>();
+
+                      return dolfin::MultiMeshDirichletBC(V, _g, subdomain, method,
+                                                 check_midpoint, exclude_overlapped_boundaries);
+                    }), py::arg("V"), py::arg("g"), py::arg("subdomain"),
+                        py::arg("method")="topological",
+                        py::arg("check_midpoint")=true,
+                        py::arg("exclude_overlapped_boundaries")=true)
+      .def(py::init<std::shared_ptr<const dolfin::MultiMeshFunctionSpace>,
+           std::shared_ptr<const dolfin::GenericFunction>,
+           std::shared_ptr<const dolfin::SubDomain>, std::string, bool, bool>(),
+           py::arg("V"), py::arg("g"), py::arg("sub_domain"),
+           py::arg("method")="topological", py::arg("check_midpoint")=true,
+           py::arg("exclude_overlapped_boundaries")=true)
+      .def(py::init<std::shared_ptr<const dolfin::MultiMeshFunctionSpace>,
+           std::shared_ptr<const dolfin::GenericFunction>,
+           std::shared_ptr<const dolfin::MeshFunction<std::size_t>>,
+           std::size_t, std::size_t, std::string>(),
+           py::arg("V"), py::arg("g"), py::arg("sub_domains"),
+           py::arg("sub_domain"), py::arg("part"), py::arg("method")="topological")
+      .def("function_space", &dolfin::MultiMeshDirichletBC::function_space)
+      .def("homogenize", &dolfin::MultiMeshDirichletBC::homogenize)
+      .def("zero", &dolfin::MultiMeshDirichletBC::zero)
+      .def("apply", (void (dolfin::MultiMeshDirichletBC::*)(dolfin::GenericVector&) const)
+           &dolfin::MultiMeshDirichletBC::apply)
+      .def("apply", (void (dolfin::MultiMeshDirichletBC::*)(dolfin::GenericMatrix&) const)
+           &dolfin::MultiMeshDirichletBC::apply)
+      .def("apply", (void (dolfin::MultiMeshDirichletBC::*)(dolfin::GenericMatrix&, dolfin::GenericVector&) const)
+           &dolfin::MultiMeshDirichletBC::apply)
+      .def("apply", (void (dolfin::MultiMeshDirichletBC::*)(dolfin::GenericVector&, const dolfin::GenericVector&) const)
+           &dolfin::MultiMeshDirichletBC::apply)
+      .def("apply", (void (dolfin::MultiMeshDirichletBC::*)(dolfin::GenericMatrix&, dolfin::GenericVector&, const dolfin::GenericVector&) const)
+           &dolfin::MultiMeshDirichletBC::apply);
 
     // dolfin::AssemblerBase
     py::class_<dolfin::AssemblerBase, std::shared_ptr<dolfin::AssemblerBase>>
       (m, "AssemblerBase")
+      .def("init_global_tensor", &dolfin::AssemblerBase::init_global_tensor)
       .def_readwrite("add_values", &dolfin::Assembler::add_values)
       .def_readwrite("keep_diagonal", &dolfin::Assembler::keep_diagonal)
       .def_readwrite("finalize_tensor", &dolfin::Assembler::finalize_tensor);
@@ -328,6 +435,7 @@ namespace dolfin_wrappers
       (m, "Form", "DOLFIN Form object")
       .def(py::init<std::shared_ptr<const ufc::form>,
                     std::vector<std::shared_ptr<const dolfin::FunctionSpace>>>())
+      .def(py::init<std::size_t, std::size_t>())
       .def("num_coefficients", &dolfin::Form::num_coefficients, "Return number of coefficients in form")
       .def("original_coefficient_position", &dolfin::Form::original_coefficient_position)
       .def("set_coefficient", (void (dolfin::Form::*)(std::size_t, std::shared_ptr<const dolfin::GenericFunction>))
@@ -342,6 +450,24 @@ namespace dolfin_wrappers
       .def("rank", &dolfin::Form::rank)
       .def("mesh", &dolfin::Form::mesh);
 
+    // dolfin::MultiMeshForm
+    py::class_<dolfin::MultiMeshForm, std::shared_ptr<dolfin::MultiMeshForm>>
+      (m, "MultiMeshForm", "DOLFIN MultiForm object")
+      .def(py::init<std::shared_ptr<const dolfin::MultiMesh>>())
+      .def(py::init<std::shared_ptr<const dolfin::MultiMeshFunctionSpace>>())
+      .def(py::init<std::shared_ptr<const dolfin::MultiMeshFunctionSpace>,
+                    std::shared_ptr<const dolfin::MultiMeshFunctionSpace>>())
+      .def("add", &dolfin::MultiMeshForm::add)
+      .def("build", &dolfin::MultiMeshForm::build)
+      .def("set_multimesh_coefficient",  &dolfin::MultiMeshForm::set_multimesh_coefficient);
+;
+
+    // dolfin::MultiMeshAssembler
+    py::class_<dolfin::MultiMeshAssembler, std::shared_ptr<dolfin::MultiMeshAssembler>>
+      (m, "MultiMeshAssembler", "DOLFIN MultiMeshAssembler object")
+      .def(py::init<>())
+      .def("assemble", &dolfin::MultiMeshAssembler::assemble);
+
     // dolfin::PointSource
     py::class_<dolfin::PointSource, std::shared_ptr<dolfin::PointSource>>
       (m, "PointSource")
@@ -453,7 +579,8 @@ namespace dolfin_wrappers
                dolfin::Variable>
       (m, "NonlinearVariationalSolver")
       .def(py::init<std::shared_ptr<dolfin::NonlinearVariationalProblem>>())
-      .def("solve", &dolfin::NonlinearVariationalSolver::solve);
+      .def("solve", &dolfin::NonlinearVariationalSolver::solve)
+      .def("default_parameters", &dolfin::NonlinearVariationalSolver::default_parameters);
 
     // dolfin::LocalSolver
     py::class_<dolfin::LocalSolver, std::shared_ptr<dolfin::LocalSolver>>
