@@ -56,6 +56,7 @@
 #include <dolfin/la/PETScOptions.h>
 #include <dolfin/la/PETScPreconditioner.h>
 #include <dolfin/la/PETScVector.h>
+#include <dolfin/la/SUNDIALSNVector.h>
 #include <dolfin/la/TpetraFactory.h>
 #include <dolfin/la/TpetraMatrix.h>
 #include <dolfin/la/TpetraVector.h>
@@ -365,7 +366,8 @@ namespace dolfin_wrappers
              std::size_t num_rows = m_range.second - m_range.first;
              std::size_t num_cols = instance.size(1);
 
-             Eigen::MatrixXd A = Eigen::MatrixXd::Zero(num_rows, num_cols);
+             typedef Eigen::Matrix<double, Eigen::Dynamic, Eigen::Dynamic, Eigen::RowMajor> EigenMatrix;
+             EigenMatrix A = EigenMatrix::Zero(num_rows, num_cols);
              std::vector<std::size_t> columns;
              std::vector<double> values;
              for (std::size_t i = 0; i < num_rows; ++i)
@@ -1075,6 +1077,18 @@ namespace dolfin_wrappers
            });
     #endif
 
+    #ifdef HAS_SUNDIALS
+    //dolfin::SUNDIALSNVector
+    py::class_<dolfin::SUNDIALSNVector, std::shared_ptr<dolfin::SUNDIALSNVector>>(m,"SUNDIALSNVector")
+      .def(py::init<const dolfin::GenericVector&>())
+      .def(py::init<const dolfin::SUNDIALSNVector&>())
+      .def(py::init<std::shared_ptr<dolfin::GenericVector>>())
+      .def("nvector", (N_Vector (dolfin::SUNDIALSNVector::*)())
+          &dolfin::SUNDIALSNVector::nvector)
+      .def("vec", (std::shared_ptr<dolfin::GenericVector> (dolfin::SUNDIALSNVector::*)())
+          &dolfin::SUNDIALSNVector::vec);
+    #endif
+
     // dolfin::VectorSpaceBasis
     py::class_<dolfin::VectorSpaceBasis, std::shared_ptr<dolfin::VectorSpaceBasis>>(m, "VectorSpaceBasis")
       .def(py::init<const std::vector<std::shared_ptr<dolfin::GenericVector>>>())
@@ -1090,10 +1104,20 @@ namespace dolfin_wrappers
           py::arg("type")="right");
 
     // la free functions
+    m.def("list_linear_algebra_backends", &dolfin::list_linear_algebra_backends);
+    m.def("list_linear_solver_methods", &dolfin::list_linear_solver_methods);
+    m.def("list_lu_solver_methods", &dolfin::list_lu_solver_methods);
+    m.def("list_krylov_solver_methods", &dolfin::list_krylov_solver_methods);
+    m.def("list_krylov_solver_preconditioners", &dolfin::list_krylov_solver_preconditioners);
     m.def("has_linear_algebra_backend", &dolfin::has_linear_algebra_backend);
-    m.def("linear_algebra_backends", &dolfin::linear_algebra_backends);
+    m.def("has_lu_solver_method", &dolfin::has_lu_solver_method);
     m.def("has_krylov_solver_method", &dolfin::has_krylov_solver_method);
     m.def("has_krylov_solver_preconditioner", &dolfin::has_krylov_solver_preconditioner);
+    m.def("linear_algebra_backends", &dolfin::linear_algebra_backends);
+    m.def("linear_solver_methods", &dolfin::linear_solver_methods);
+    m.def("lu_solver_methods", &dolfin::lu_solver_methods);
+    m.def("krylov_solver_methods", &dolfin::krylov_solver_methods);
+    m.def("krylov_solver_preconditioners", &dolfin::krylov_solver_preconditioners);
     m.def("normalize", &dolfin::normalize, py::arg("x"), py::arg("normalization_type")="average");
 
     // la/solve free functions
