@@ -69,7 +69,9 @@ namespace dolfin_wrappers
           });
 
     // dolfin::CellType
-    py::class_<dolfin::CellType> celltype(m, "CellType");
+    py::class_<dolfin::CellType>
+      celltype(m, "CellType");
+    celltype.def("normal", (dolfin::Point (dolfin::CellType::*)(const dolfin::Cell&, std::size_t) const) &dolfin::CellType::normal);
 
     // dolfin::CellType enums
     py::enum_<dolfin::CellType::Type>(celltype, "Type")
@@ -89,7 +91,9 @@ namespace dolfin_wrappers
     py::class_<dolfin::MeshGeometry, std::shared_ptr<dolfin::MeshGeometry>>
       (m, "MeshGeometry", "DOLFIN MeshGeometry object")
       .def("dim", &dolfin::MeshGeometry::dim, "Geometrical dimension")
-      .def("degree", &dolfin::MeshGeometry::degree, "Degree");
+      .def("degree", &dolfin::MeshGeometry::degree, "Degree")
+      .def("get_entity_index", &dolfin::MeshGeometry::get_entity_index)
+      .def("num_entity_coordinates", &dolfin::MeshGeometry::num_entity_coordinates);
 
     // dolfin::MeshTopology class
     py::class_<dolfin::MeshTopology, std::shared_ptr<dolfin::MeshTopology>, dolfin::Variable>
@@ -99,7 +103,7 @@ namespace dolfin_wrappers
       .def("init", (void (dolfin::MeshTopology::*)(std::size_t, std::size_t, std::size_t))
            &dolfin::MeshTopology::init)
       .def("__call__", (const dolfin::MeshConnectivity& (dolfin::MeshTopology::*)(std::size_t, std::size_t) const)
-           &dolfin::MeshTopology::operator())
+           &dolfin::MeshTopology::operator(), py::return_value_policy::reference_internal)
       .def("size", &dolfin::MeshTopology::size)
       .def("hash", &dolfin::MeshTopology::hash)
       .def("init_global_indices", &dolfin::MeshTopology::init_global_indices)
@@ -173,6 +177,7 @@ namespace dolfin_wrappers
       .def("num_faces", &dolfin::Mesh::num_faces, "Number of faces")
       .def("num_facets", &dolfin::Mesh::num_facets, "Number of facets")
       .def("num_cells", &dolfin::Mesh::num_cells, "Number of cells")
+      .def("order", &dolfin::Mesh::order)
       .def("ordered", &dolfin::Mesh::ordered)
       .def("rmax", &dolfin::Mesh::rmax)
       .def("rmin", &dolfin::Mesh::rmin)
@@ -180,6 +185,7 @@ namespace dolfin_wrappers
            &dolfin::Mesh::rotate)
       .def("rotate", (void (dolfin::Mesh::*)(double, std::size_t)) &dolfin::Mesh::rotate,
                       py::arg("angle"), py::arg("axis")=2)
+      .def("scale", &dolfin::Mesh::scale)
       .def("num_entities_global", &dolfin::Mesh::num_entities_global)
       .def("smooth", &dolfin::Mesh::smooth, py::arg("num_iterations")=1)
       .def("smooth_boundary", &dolfin::Mesh::smooth_boundary)
@@ -217,14 +223,20 @@ namespace dolfin_wrappers
     py::class_<dolfin::BoundaryMesh, std::shared_ptr<dolfin::BoundaryMesh>, dolfin::Mesh>
       (m, "BoundaryMesh", "DOLFIN BoundaryMesh object")
       .def(py::init<const dolfin::Mesh&, std::string, bool>(),
-           py::arg("mesh"), py::arg("type"), py::arg("order")=true);
+           py::arg("mesh"), py::arg("type"), py::arg("order")=true)
+      .def("entity_map", (dolfin::MeshFunction<std::size_t>& (dolfin::BoundaryMesh::*)(std::size_t))
+           &dolfin::BoundaryMesh::entity_map)
+      .def("entity_map", (const dolfin::MeshFunction<std::size_t>& (dolfin::BoundaryMesh::*)(std::size_t) const)
+           &dolfin::BoundaryMesh::entity_map);
 
     // dolfin::MeshConnectivity class
     py::class_<dolfin::MeshConnectivity, std::shared_ptr<dolfin::MeshConnectivity>>
       (m, "MeshConnectivity", "DOLFIN MeshConnectivity object")
       .def("__call__", [](const dolfin::MeshConnectivity& self, std::size_t i)
            {
-             return Eigen::Map<const Eigen::Matrix<unsigned int, Eigen::Dynamic, 1>>(self(i), self.size(i));})
+             return Eigen::Map<const Eigen::Matrix<unsigned int, Eigen::Dynamic, 1>>(self(i), self.size(i));
+           }, py::return_value_policy::reference_internal)
+      .def("__call__", (const std::vector<unsigned int>& (dolfin::MeshConnectivity::*)() const)&dolfin::MeshConnectivity::operator(), py::return_value_policy::reference_internal)
       .def("size", (std::size_t (dolfin::MeshConnectivity::*)() const)
            &dolfin::MeshConnectivity::size)
       .def("size", (std::size_t (dolfin::MeshConnectivity::*)(std::size_t) const)
@@ -258,7 +270,8 @@ namespace dolfin_wrappers
     py::class_<dolfin::Vertex, std::shared_ptr<dolfin::Vertex>, dolfin::MeshEntity>
       (m, "Vertex", "DOLFIN Vertex object")
       .def(py::init<const dolfin::Mesh&, std::size_t>())
-      .def("point", &dolfin::Vertex::point);
+      .def("point", &dolfin::Vertex::point)
+      .def("x", (double (dolfin::Vertex::*)(std::size_t) const) &dolfin::Vertex::x );
 
     // dolfin::Edge
     py::class_<dolfin::Edge, std::shared_ptr<dolfin::Edge>, dolfin::MeshEntity>
@@ -280,8 +293,8 @@ namespace dolfin_wrappers
       (m, "Facet", "DOLFIN Facet object")
       .def(py::init<const dolfin::Mesh&, std::size_t>())
       .def("exterior", &dolfin::Facet::exterior)
-      .def("normal", (dolfin::Point (dolfin::Facet::*)() const)  &dolfin::Facet::normal);
-
+      .def("normal", (dolfin::Point (dolfin::Facet::*)() const)  &dolfin::Facet::normal)
+      .def("normal", (double (dolfin::Facet::*)(std::size_t) const) &dolfin::Facet::normal);
     // dolfin::Cell
     py::class_<dolfin::Cell, std::shared_ptr<dolfin::Cell>, dolfin::MeshEntity>
       (m, "Cell", "DOLFIN Cell object")
@@ -290,6 +303,7 @@ namespace dolfin_wrappers
       .def("collides", (bool (dolfin::Cell::*)(const dolfin::MeshEntity&) const) &dolfin::Cell::collides)
       .def("contains", &dolfin::Cell::contains)
       .def("distance", &dolfin::Cell::distance)
+      .def("cell_normal", &dolfin::Cell::cell_normal)
       .def("facet_area", &dolfin::Cell::facet_area)
       .def("h", &dolfin::Cell::h)
       .def("inradius", &dolfin::Cell::inradius)
@@ -392,6 +406,10 @@ namespace dolfin_wrappers
       .def("mesh", &dolfin::MeshFunction<SCALAR>::mesh) \
       .def("set_values", &dolfin::MeshFunction<SCALAR>::set_values) \
       .def("set_all", &dolfin::MeshFunction<SCALAR>::set_all) \
+      .def("set_value", (void (dolfin::MeshFunction<SCALAR>::*)(std::size_t, const SCALAR&)) \
+	   &dolfin::MeshFunction<SCALAR>::set_value) \
+      .def("set_value", (void (dolfin::MeshFunction<SCALAR>::*)(std::size_t, const SCALAR&, const dolfin::Mesh&)) \
+	   &dolfin::MeshFunction<SCALAR>::set_value) \
       .def("where_equal", &dolfin::MeshFunction<SCALAR>::where_equal) \
       .def("array", [](dolfin::MeshFunction<SCALAR>& self) \
            { return Eigen::Map<Eigen::Matrix<SCALAR, Eigen::Dynamic, 1>>(self.values(), self.size()); })
@@ -455,6 +473,7 @@ namespace dolfin_wrappers
     // dolfin::MeshQuality
     py::class_<dolfin::MeshQuality>
       (m, "MeshQuality", "DOLFIN MeshQuality class")
+      .def_static("aspect_ratio_gamma", &dolfin::MeshQuality::aspect_ratio_gamma)
       .def_static("radius_ratios", &dolfin::MeshQuality::radius_ratios)
       .def_static("radius_ratio_histogram_data", &dolfin::MeshQuality::radius_ratio_histogram_data)
       .def_static("radius_ratio_min_max", &dolfin::MeshQuality::radius_ratio_min_max)
@@ -512,6 +531,7 @@ namespace dolfin_wrappers
     // dolfin::PeriodicBoundaryComputation
     py::class_<dolfin::PeriodicBoundaryComputation>
       (m, "PeriodicBoundaryComputation")
+      .def(py::init<>())
       .def_static("compute_periodic_pairs", &dolfin::PeriodicBoundaryComputation::compute_periodic_pairs)
       .def_static("masters_slaves", &dolfin::PeriodicBoundaryComputation::masters_slaves);
 
@@ -525,12 +545,13 @@ namespace dolfin_wrappers
     py::class_<dolfin::MeshTransformation>(m, "MeshTransformation")
       .def_static("translate", &dolfin::MeshTransformation::translate)
       .def_static("rescale", &dolfin::MeshTransformation::rescale)
+      .def_static("scale", &dolfin::MeshTransformation::scale)
       .def_static("rotate", (void (*)(dolfin::Mesh&, double, std::size_t)) &dolfin::MeshTransformation::rotate)
       .def_static("rotate", (void (*)(dolfin::Mesh&, double, std::size_t, const dolfin::Point&))
                   &dolfin::MeshTransformation::rotate);
 
     py::class_<dolfin::MultiMesh, std::shared_ptr<dolfin::MultiMesh>,
-	       dolfin::Variable>(m, "MultiMesh")
+	       dolfin::Variable>(m, "MultiMesh", py::dynamic_attr())
       .def(py::init<>())
       .def("add", &dolfin::MultiMesh::add)
       .def("build", &dolfin::MultiMesh::build, py::arg("quadrature_order") = 2)
@@ -540,6 +561,13 @@ namespace dolfin_wrappers
       .def("cut_cells", &dolfin::MultiMesh::cut_cells)
       .def("uncut_cells", &dolfin::MultiMesh::uncut_cells)
       .def("covered_cells", &dolfin::MultiMesh::covered_cells)
+      .def("mark_covered", &dolfin::MultiMesh::mark_covered, "Function that marks a set of cells, given by indicies in a list, as covered.")
+      .def("compute_area", &dolfin::MultiMesh::compute_area)
+      .def("auto_cover", &dolfin::MultiMesh::auto_cover, " Marks all uncut and cut cells connected to the given point as covered.")
+      .def("quadrature_rules_interface",
+	   (const std::map<unsigned int, std::vector<dolfin::MultiMesh::quadrature_rule> >&(dolfin::MultiMesh::*)(std::size_t) const)(&dolfin::MultiMesh::quadrature_rules_interface))
+      .def("quadrature_rules_interface",
+	   static_cast<const std::vector<dolfin::MultiMesh::quadrature_rule>(dolfin::MultiMesh::*)(std::size_t, unsigned int) const>(&dolfin::MultiMesh::quadrature_rules_interface))
       .def("quadrature_rules_cut_cells",
 	   static_cast<const dolfin::MultiMesh::quadrature_rule(dolfin::MultiMesh::*)(std::size_t, unsigned int) const>(&dolfin::MultiMesh::quadrature_rules_cut_cells));
   }
